@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -51,7 +50,7 @@ static void nwrat_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_radio_settings_rat_mode_query_cb_t cb = cbd->cb;
-	enum ofono_radio_access_mode mode;
+	unsigned int mode;
 	struct ofono_error error;
 	GAtResultIter iter;
 	int value;
@@ -118,8 +117,7 @@ static void nwrat_modify_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, cbd->data);
 }
 
-static void nw_set_rat_mode(struct ofono_radio_settings *rs,
-				enum ofono_radio_access_mode mode,
+static void nw_set_rat_mode(struct ofono_radio_settings *rs, unsigned int mode,
 				ofono_radio_settings_rat_mode_set_cb_t cb,
 				void *data)
 {
@@ -157,8 +155,10 @@ static void nwrat_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_radio_settings *rs = user_data;
 
-	if (!ok)
+	if (!ok) {
+		ofono_radio_settings_remove(rs);
 		return;
+	}
 
 	ofono_radio_settings_register(rs);
 }
@@ -193,7 +193,7 @@ static void nw_radio_settings_remove(struct ofono_radio_settings *rs)
 	g_free(rsd);
 }
 
-static struct ofono_radio_settings_driver driver = {
+static const struct ofono_radio_settings_driver driver = {
 	.name			= "nwmodem",
 	.probe			= nw_radio_settings_probe,
 	.remove			= nw_radio_settings_remove,
