@@ -36,6 +36,7 @@
 #include "simutil.h"
 #include "util.h"
 #include "storage.h"
+#include "missing.h"
 
 #define SETTINGS_STORE "netreg"
 #define SETTINGS_GROUP "Settings"
@@ -733,7 +734,7 @@ static gboolean update_operator_list(struct ofono_netreg *netreg, int total,
 			/* New operator */
 			struct network_operator_data *opd;
 
-			opd = g_memdup(copd,
+			opd = g_memdup2(copd,
 					sizeof(struct network_operator_data));
 
 			if (!network_operator_dbus_register(netreg, opd)) {
@@ -1351,8 +1352,8 @@ void ofono_netreg_status_notify(struct ofono_netreg *netreg, int status,
 	if (netreg == NULL)
 		return;
 
-	DBG("%s status %d tech %d", __ofono_atom_get_path(netreg->atom),
-							status, tech);
+	DBG("%s status %d tech %d lac %d ci %d",
+	    __ofono_atom_get_path(netreg->atom), status, tech, lac, ci);
 
 	if (netreg->status != status) {
 		struct ofono_modem *modem;
@@ -1407,6 +1408,11 @@ void ofono_netreg_time_notify(struct ofono_netreg *netreg,
 	if (info == NULL)
 		return;
 
+	DBG("net time %d-%02d-%02d %02d:%02d:%02d utcoff %d dst %d",
+	    info->year, info->mon, info->mday,
+	    info->hour, info->min, info->sec,
+	    info->utcoff, info->dst);
+
 	__ofono_nettime_info_received(modem, info);
 }
 
